import { CommonModule } from "@angular/common";
import { Component } from "@angular/core";
import { ReactiveFormsModule, FormControl } from "@angular/forms";
import { RouterModule } from "@angular/router";
import { ConsentComponent } from "../../consent/consent.component.js";
import { ErrorComponent } from "../../error/error.component.js";
import { CheckboxComponent } from "../../inputs/checkbox/checkbox.component.js";
import { MultiLineComponent } from "../../inputs/multi-line/multi-line.component.js";
import { SelectMenuComponent } from "../../inputs/select-menu/select-menu.component.js";
import { SingleLineComponent } from "../../inputs/single-line/single-line.component.js";
import { SuccessComponent } from "../../success/success.component.js";
import { UserinfoComponent } from "../../userinfo/userinfo.component.js";
import { ApiService } from "../../api.service.js";
import type { Platform } from "@repo/types/prod/unions/platform.js";
import type { Sanction } from "@repo/types/prod/unions/sanction.js";

/**
 *
 */
@Component({
  imports: [
    RouterModule,
    CommonModule,
    ConsentComponent,
    UserinfoComponent,
    ErrorComponent,
    ReactiveFormsModule,
    CheckboxComponent,
    SingleLineComponent,
    SelectMenuComponent,
    MultiLineComponent,
    SuccessComponent,
  ],
  selector:    'appeal-form',
  styleUrl:    './appeal.component.css',
  templateUrl: "./appeal.component.html",
})
export class AppealComponent {
  public loading = false;
  public error = "";
  public success = false;

  public firstName = new FormControl("");
  public lastName = new FormControl("");
  public email = new FormControl("");
  public company = new FormControl("");

  public understandBinding = new FormControl(false);
  public sanctionType = new FormControl("reminder");
  public caseNumber = new FormControl("");
  public sanctionPlatform = new FormControl<Platform>("Forum");
  public platformUsername = new FormControl("");
  public sanctionReason = new FormControl("");
  public sanctionFair = new FormControl("");
  public behaviourViolation = new FormControl("");
  public appealReason = new FormControl("");
  public behaviourImprove = new FormControl("");

  public consent = new FormControl(false);

  /**
   * @param apiService
   */
  constructor(private readonly apiService: ApiService) {}

  /**
   * @param e
   */
  public submit(e: MouseEvent): void {
    this.error = "";
    const { form } = e.target as HTMLButtonElement;
    const valid = form?.reportValidity();
    if (!valid) {
      return;
    }
    this.loading = true;

    this.apiService.
      submitAppeal({
        caseNumber:         Number.parseInt(this.caseNumber.value ?? "0", 10),
        email:              this.email.value ?? undefined,
        appealReason:       this.appealReason.value ?? undefined,
        firstName:          this.firstName.value ?? undefined,
        behaviourImprove:   this.behaviourImprove.value ?? undefined,
        lastName:         this.lastName.value ?? undefined,
        behaviourViolation: this.behaviourViolation.value ?? undefined,
        platformUsername: this.platformUsername.value ?? undefined,
        consent:            this.consent.value ?? false,
        sanctionFair:     this.sanctionFair.value ?? undefined,
        sanctionPlatform:   this.sanctionPlatform.value ?? undefined,
        sanctionType:
          (this.sanctionType.value?.split(/\s+/)[0] as Sanction | undefined)
          ?? undefined,
        sanctionReason:    this.sanctionReason.value ?? undefined,
        understandBinding: this.understandBinding.value ?? undefined,
      }).
      then((res) => {
        if ("error" in res) {
          this.error = res.error;
          this.loading = false;
        } else {
          this.error = "";
          this.success = true;
        }
      }).
      catch(() => {
        this.error = "An error occurred while submitting the form.";
        this.loading = false;
      });
  }
}
